## @file CurveT.py
#  @author Steven Palmer and Henry Madej
#  @brief CurveT
#  @date 20/02/2018

from Exceptions import *
from SeqServices import *
from scipy.interpolate import interp1d



## @brief An abstract data type that represents a Curve
class CurveT:

  MAX_ORDER = 2

  ## @brief 
  #  @details 
  def __init__(self, X, Y, i):
    if not isAscending(X):
      raise IndepVarNotAscending
    if i < 1 or i > CurveT.MAX_ORDER:
      raise InvalidInterpOrder
      
    self.__minx = X[0] 
    self.__maxx = X[-1]
    self.__o = i
    
    if self.__o == 1:
      self.__f = interp1d(X, Y, 'linear')
    elif self.__o == 2:
      self.__f = interp1d(X, Y, 'quadratic')
  
  ## @brief 
  #  @details 
  def minD(self):
    return self.__minx
  
  ## @brief 
  #  @details 
  def maxD(self):
    return self.__maxx
    
  ## @brief 
  #  @details 
  def order(self):
    return self.__o
  
  ## @brief 
  #  @details 
  def eval(self, x):
    if x < self.__minx or x > self.__maxx:
      raise OutOfDomain
    return float(self.__f(x))

  ## @brief 
  #  @details 
  def dfdx(self, x):
    if x < self.__minx or x > self.__maxx:
      raise OutOfDomain
    return self.__f(x + DX) - self.__f(x) / DX

  ## @brief 
  #  @details 
  def d2fdx2(self, x):
    if x < self.__minx or x > self.__maxx:
      raise OutOfDomain
    return self.__f(x + 2*DX) - 2*self.__f(x + DX) + self.__f(x) / DX**2