## @file SeqServices.py
#  @author Steven Palmer and Henry Madej
#  @brief Provides the services for working with sequences
#  @date 20/02/2018


## @brief isAscending checks if a sequence is in ascending order
#  @param X sequence to be checked for ascending order
#  @return boolean False if not in ascending order True otherwise
def isAscending(X):
  for i in range(0, len(X) - 1):
    if X[i + 1] < X[i]:
      return False
  
  return True
  
## @brief isInBounds checks if value (x) is within bounds of sequence (X)
#  @param X sequence to be checked against
#  @param x value to be checked for in bounds 
#  @return boolean False if not in bounds True otherwise
def isInBounds(X, x):
  return X[0] <= x and x <= X[len(X) - 1]

## @brief interpLin given two points interpolates the y value for a given x
#  @details linear interpolation
#  @param x1 x value of the first point
#  @param y1 y value of the first point
#  @param x2 x value of the second point
#  @param y2 y value of the second point
#  @param x value for which we want to interpolate a y value
#  @return y value for given x value
def interpLin(x1, y1, x2, y2, x):
  return (y2 - y1) / (x2 - x1) * (x - x1) + y1
  
## @brief interpQuad given three points interpolates the y value for given x
#  @details quadratic interpolation
#  @param x0 x value of the first point
#  @param y0 y value of the first point
#  @param x1 x value of the second point
#  @param y1 y value of the second point
#  @param x2 x value of the third point
#  @param y2 y value of the third point
#  @param x value for which we want to interpolate a y value
#  @return y value for given x value
def interpQuad(x0, y0, x1, y1, x2, y2, x):
  return y1 + (y2 - y0) / (x2 - x0) * (x - x1) + (y2 - 2*y1 + y0) / (2*(x2 - x1)**2) * (x - x1)**2
  
## @brief index given a sequence and value, returns index of that value if in 
#  sequence
#  @details Seq[i] <= x <= Seq[i+1]
#  @param X sequence to be searched
#  @param x value to look for
#  @return index of closest value to given x in sequence
def index(X, x):
  for i in range(0, len(X) - 1):
    if X[i] <= x and x < X[i+1]:
      return i
