from CurveADT import *
import Data
import Load
import Plot
import Exceptions
import SeqServices

from pytest import *

class TestCurveADT:
  def test_minD(self):
    curve = CurveT([0, 50, 99], [1, 2, 3], 1)
    assert curve.minD() == 0

  def test_maxD(self):
    curve = CurveT([0, 50, 99], [1, 2, 3], 1)
    assert curve.maxD() == 99
    
  def test_order(self):
    curve = CurveT([0, 50, 99], [1, 2, 3], 1)
    assert curve.order() == 1

  def test_raises_indepVarNotAscending(self):
    with raises(Exceptions.IndepVarNotAscending):
      CurveT([ 0, -1 ], [ 0, 0 ], 1)
    
  def test_raises_SeqSizeMismatch(self):
    with raises(Exceptions.SeqSizeMismatch):
      CurveT([ 0 ], [ 0, 0 ], 1)

  def test_raises_OutOfDomain_eval_below(self):
    curve = CurveT([ 0, 1, 2 ], [ 0, 0, 0 ], 1)
    with raises(Exceptions.OutOfDomain):
      curve.eval(-1)

  def test_raises_OutOfDomain_eval_above(self):
    curve = CurveT([ 0, 1, 2 ], [ 0, 0, 0 ], 1)
    with raises(Exceptions.OutOfDomain):
      curve.eval(3)

  def test_raises_OutOfDomain_dfdx_below(self):
    curve = CurveT([ 0, 1, 2 ], [ 0, 0, 0 ], 1)
    with raises(Exceptions.OutOfDomain):
      curve.dfdx(-1)

  def test_raises_OutOfDomain_dfdx_above(self):
    curve = CurveT([ 0, 1, 2 ], [ 0, 0, 0 ], 1)
    with raises(Exceptions.OutOfDomain):
      curve.dfdx(3)

  def test_raises_OutOfDomain_d2fdx2_below(self):
    curve = CurveT([ 0, 1, 2 ], [ 0, 0, 0 ], 1)
    with raises(Exceptions.OutOfDomain):
      curve.d2fdx2(-1)

  def test_raises_OutOfDomain_d2fdx2_above(self):
    curve = CurveT([ 0, 1, 2 ], [ 0, 0, 0 ], 1)
    with raises(Exceptions.OutOfDomain):
      curve.d2fdx2(3)

  def test_eval_linear(self):
    curve = CurveT([0, 1, 3], [0, 1, 3], 1)
    assert curve.eval(2) == approx(2, 0.1)
    
  def test_eval_quadratic(self):
    curve = CurveT([0, 1, 2], [1, 0, 1], 2)
    assert curve.eval(0.5) == approx(1.25, 0.1)

  def test_dfdx(self):
    curve = CurveT([0, 1], [0, 1000], 1)
    assert curve.dfdx(0) == approx(1, 0.0001)

  def test_dfdx_2(self):
    curve = CurveT([0, 1], [0, 1], 1)
    assert curve.dfdx(0) == approx(0.001, 0.0001)

  def test_d2fdx2(self):
    curve = CurveT([0, 1], [0, 1], 1)
    assert curve.d2fdx2(0) == approx(0, 0.0001)

  def test_d2fdx2_2(self):
    curve = CurveT([0, 1], [0, 1000], 1)
    assert curve.d2fdx2(0) == approx(0, 0.0001)

class TestData:
  def test_constructor_S(self):
    Data.Data.init()
    assert Data.Data.S == []

  def test_constructor_Z(self):
    Data.Data.init()
    assert Data.Data.Z == []

  def test_constructor_MAX(self):
    Data.Data.init()
    assert Data.Data.MAX_SIZE == 10

  def test_add(self):
    Data.Data.init()
    test_curve_a = CurveT([0,1,2], [0,1,2], 1)
    test_curve_b = CurveT([0,1,2], [0,1,2], 1)
    Data.Data.add(test_curve_a, 0)
    Data.Data.add(test_curve_b, 1)
    assert Data.Data.S[0] == test_curve_a
    assert Data.Data.S[1] == test_curve_b

  def test_add_full_exception(self):
    Data.Data.init()
    test_curve_a = CurveT([0,1,2], [0,1,2], 1)
    with raises(Full):
      for x in range(11):
        Data.Data.add(test_curve_a, x)

  def test_getC(self):
    Data.Data.init()
    test_curve_a = CurveT([0,1,2], [0,1,2], 1)
    test_curve_b = CurveT([0,1,2], [0,1,2], 1)
    Data.Data.add(test_curve_a, 0)
    Data.Data.add(test_curve_b, 1)
    assert Data.Data.getC(0) == test_curve_a
    assert Data.Data.getC(1) == test_curve_b

  def test_getC_empty(self):
    Data.Data.init()
    with raises(InvalidIndex): 
      Data.Data.getC(0)

  def test_getC_invalid_index_below(self):
    Data.Data.init()
    test_curve_a = CurveT([0,1,2], [0,1,2], 1)
    test_curve_b = CurveT([0,1,2], [0,1,2], 1)
    Data.Data.add(test_curve_a, 0)
    Data.Data.add(test_curve_b, 1)
    with raises(InvalidIndex):
      Data.Data.getC(-1)

  def test_getC_invalid_index_above(self):
    Data.Data.init()
    test_curve_a = CurveT([0,1,2], [0,1,2], 1)
    test_curve_b = CurveT([0,1,2], [0,1,2], 1)
    Data.Data.add(test_curve_a, 0)
    Data.Data.add(test_curve_b, 1)
    with raises(InvalidIndex):
      Data.Data.getC(2)

  def test_add_IndepVarNotAscending_exception(self):
    Data.Data.init()
    test_curve_a = CurveT([-1], [-1], 1)
    Data.Data.add(test_curve_a, 3)
    with raises(IndepVarNotAscending):
      Data.Data.add(test_curve_a, 1)

  def test_eval(self):
    Data.Data.init()
    test_curve_a = CurveT([0,1,2,3], [0,1,2,3], 1)
    test_curve_b = CurveT([0,1,2,3], [1,2,3,4], 1)
    Data.Data.add(test_curve_a, 0)
    Data.Data.add(test_curve_b, 1)
    assert Data.Data.eval(1, 0) == approx(1, 0.005)

  def test_eval_OutOfDomain_exception(self):
    Data.Data.init()
    test_curve_a = CurveT([0,1,2,3], [0,1,2,3], 1)
    test_curve_b = CurveT([0,1,2,3], [1,2,3,4], 1)
    Data.Data.add(test_curve_a, 0)
    Data.Data.add(test_curve_b, 1)
    with raises(OutOfDomain):
      Data.Data.eval(0, 4)

  def test_slice(self):
    Data.Data.init()
    test_curve_a = CurveT([0,1,2,3], [0,1,2,3], 1)
    test_curve_b = CurveT([0,1,2,3], [1,2,3,4], 1)
    Data.Data.add(test_curve_a, 0)
    Data.Data.add(test_curve_b, 1)
    new_curve = Data.Data.slice(1, 1)
    assert new_curve.minD()  == 0
    assert new_curve.maxD()  == 1
    assert new_curve.order() == 1

class TestSeqServices:
  def test_index(self):
    assert SeqServices.index([1,2,3,4,5,6], 3.5) == 2
    assert SeqServices.index([1,2], 1.5) == 0

  def test_is_ascending_empty(self):
    assert SeqServices.isAscending([])

  def test_isAscending_unit(self):
    assert SeqServices.isAscending([ 1 ])

  def test_isAscending_two(self):
    assert SeqServices.isAscending([ 1, 2 ])

  def test_isAscending_repeat(self):
    assert SeqServices.isAscending([ 0, 0, 0 ])

  def test_isAscending_descending(self):
    assert not SeqServices.isAscending([ 2, 1, 0 ])

  def test_isInBounds(self):
    assert SeqServices.isInBounds([ 1, 2 ], 1.5)
    assert SeqServices.isInBounds([ 1, 2 ], 2)
    assert SeqServices.isInBounds([ 1 ], 1)

  def test_isInBounds_below(self):
    assert not SeqServices.isInBounds([ 1, 2 ], 0.9999)

  def test_isInBounds_In(self):
    assert SeqServices.isInBounds([ 1, 2 ], 1.000)
    assert SeqServices.isInBounds([ 1, 2 ], 1.00001)
    assert SeqServices.isInBounds([ 1, 2 ], 1.9999)
    assert SeqServices.isInBounds([ 1, 2 ], 1.5)

  def test_isInBounds_above(self):
    assert not SeqServices.isInBounds([ 1, 2 ], 2.0001)

  def test_isInBounds_negative(self):
    assert not SeqServices.isInBounds([ 1, 2 ], -1)
    assert not SeqServices.isInBounds([ 1, 2 ], -2)

  def test_interpLin(self):
    assert SeqServices.interpLin(0, 1, 1, 2, 0.5) == 1.5

  def test_interpQuad(self):
    # -3.5 + 2.875x + 1.01786x^2
    assert SeqServices.interpQuad(0, -3.5, 1, 0.39286, 2, 6.32144, 3) == approx(14.2857, 0.0001)
  