#include "LineADT.h"

#include "PointADT.h"
#include "MapTypes.h"
#include "Exceptions.h"

LineT::LineT(PointT s, CompassT o, nat l) : s(s)
{
  if(l == 0)
    throw invalid_argument();
  
  this->o = o;
  this->L = l;
}

PointT LineT::strt() const
{
  return s; // returning s makes a copy
}

PointT LineT::end() const
{
  nat d = L - 1;
  
  switch(o) {
    case N:
      return PointT(s.x(), s.y() + d);
    case S:
      return PointT(s.x(), s.y() - d);
    case E:
      return PointT(s.x() + d, s.y());
    case W:
      return PointT(s.x() - d, s.y());
  }
}

CompassT LineT::orient() const
{
  return o;
}

nat LineT::len() const
{
  return L;
}

LineT LineT::flip() const
{
  switch(o) {
    case N:
      return LineT(s, S, L);
    case S:
      return LineT(s, N, L);
    case E:
      return LineT(s, W, L);
    case W:
      return LineT(s, E, L);
  }
}

LineT LineT::rotate(RotateT r) const
{
  switch(o) {
    case N:
      return (r == CW) ? LineT(s, E, L) : LineT(s, W, L);
    case S:
      return (r == CW) ? LineT(s, W, L) : LineT(s, E, L);
    case E:
      return (r == CW) ? LineT(s, S, L) : LineT(s, N, L);
    case W:
      return (r == CW) ? LineT(s, N, L) : LineT(s, S, L);
  }
}

LineT LineT::translate(int dx, int dy) const
{
  return LineT(s.translate(dx, dy), o, L);
}
