/**
 * \file CardADT.h
 * \brief Provides the CardADT class for representing a card in a standard 52-card deck of cards.
 * \author Christopher W. Schankula, MacID: schankuc
 * \date April 8th, 2018
*/


#ifndef CARDADT_H
#define CARDADT_H

#include "CardTypes.h"
#include <map>

/**
 *  \brief Class for representing a card in a deck of cards given a suit and a rank.
 *  \details Implements a class for representing a card in a deck of cards. Stores the 
 suit and the rank of the card. Also provides functions for outputting the rank, suit
 and colour of the card.
 */
class CardT {
	private:
		SuitT suit;
		RankT rank;

	public:
		/**
		 *  \brief Creates a new card
		 *  \details Create a new card given a suit and a rank for the card.
		 *  \param s The suit of the card (H, D, C, S).
		 *  \param r The rank of the card (A,2-10,J,Q,K).
		 */
		CardT (SuitT s, RankT r);

		/**
		 *  \brief Returns the suit of the current card.
		 *  \return The suit of the given card.
		 */
		SuitT getSuit();

		/**
		 *  \brief Returns the colour of the current card.
		 *  \return The colour of the given card.
		 */
		ColourT getColour();

		/**
		 *  \brief Returns the rank of the current card.
		 *  \return The rank of the given card.
		 */
		RankT getRank();
};

#endif