/**
 * \file CardDeckUtils.h
 * \brief A module providing functions to generate and randomize sequences representing decks of cards.
 * \author Christopher W. Schankula, MacID: schankuc
 * \date April 8th, 2018
*/

#ifndef CARDDECKUTILS_H
#define CARDDECKUTILS_H

#include <vector>
#include "CardStack.h"
#include "GameBoardTypes.h"

/**
 *  \brief A static class for generating and shuffling sequences of playing cards, 
 * representing a deck of cards.
 */
class CardDeckUtils{
	private:
		//private to avoid instances of this class being created.
		CardDeckUtils();
		static std::vector<CardT> deckSet();
		static std::vector<CardT> permute(std::vector<CardT> inSet);

	public:
		/**
		 *  \brief Returns a random sequence of CardT object, representing a shuffled deck of cards.
		 *  \returns A sequence of all 52 playing cards in a randomized order.
		 */
		static std::vector<CardT> randomDeck();
};

#endif