/**
 * \file Stack.h
 * \brief The implementation of the generic stack module described in Stack.h.
 * \author Christopher W. Schankula, MacID: schankuc
 * \date April 8th, 2018
*/

#include "Stack.h"
#include "CardADT.h"
#include "Exceptions.h"
#include <vector>

template <class T>
Stack<T>::Stack(unsigned int n, std::vector<T> s){
	if (s.size() > n) throw invalid_argument();
	if (n == 0) throw invalid_argument();

	this->S = s;
	this->max_size = n;
}

template <class T>
Stack<T> Stack<T>::push(T e){
	if (this->max_size == this->S.size()) throw stack_full();

	std::vector<T> new_(this->S);
	new_.push_back(e);
	return Stack<T>(this->max_size, new_);
}

template <class T>
Stack<T> Stack<T>::pop(){
	if (this->S.size() == 0) throw stack_empty();

	std::vector<T> new_(this->S);
	new_.pop_back();
	return Stack<T>(this->max_size, new_);
}

template <class T>
T Stack<T>::top(){
	if (this->S.size() == 0) throw stack_empty();

	return this->S.back();
}

template <class T>
unsigned int Stack<T>::getSize(){
	return this->S.size();
}

template <class T>
unsigned int Stack<T>::getMaxSize(){
	return this->max_size;
}

template <class T>
std::vector<T> Stack<T>::toSeq(){
	return this->S;
}

//A stack of CardT objects
template class Stack<CardT>;