/**
 * \file testCardADT.cpp
 * \brief Implementation of CardDeckUtils tests.
 * \author Christopher W. Schankula, MacID: schankuc
 * \date April 9th, 2018
*/

#include "catch.h"
#include <vector>
#include "CardDeckUtils.h"

TEST_CASE( "tests for CardDeckUtils" , "[CardDeckUtils]" ) {
	SECTION("test randomDeck() contains all cards"){
		std::vector<CardT> deck = CardDeckUtils::randomDeck();
		REQUIRE(deck.size() == 52);

		bool found[52];

		for(int i = 0; i < 52; i++)
			found[i] = false;

		//mark cards as found in found array
		for (auto card: deck)
			found[card.getSuit()*13 + card.getRank()] = true;

		//ensure that all cards are in the found array
		for (auto f: found)
			REQUIRE(f);
	}

	SECTION("test randomDeck() is not the same ordering as the original deck"){
		//generate deck same way that CardDeckUtils generates it in the deckSet function
		RankT ranks[] = {A, C2, C3, C4, C5, C6, C7, C8, C9, C10, J, Q, K};
		SuitT suits[] = {H, D, S, C};

		std::vector<CardT> deck = std::vector<CardT>();

		for (int i = 0; i < 4; i++)
			for (int j = 0; j < 13; j++)
				deck.push_back(CardT(suits[i], ranks[j]));

		std::vector<CardT> deck0 = CardDeckUtils::randomDeck();

		//check that the shuffled deck is not the same as the original one (SUPER low chance that this could happen randomly)
		bool same = true;
		for (int i = 0; i < 52; i++)
			same = same && (deck[i].getSuit() == deck0[i].getSuit() && deck[i].getRank() == deck0[i].getRank());
		REQUIRE(!same);
	}
}
