/**
 * \file testCardADT.cpp
 * \brief Implementation of CardT tests.
 * \author Christopher W. Schankula, MacID: schankuc
 * \date March 26th, 2018
*/

#include "catch.h"
#include "CardStack.h"
#include "Exceptions.h"

TEST_CASE( "tests for Stack" , "[Stack]" ) {
	CardStackT cs(4, std::vector<CardT>());

	SECTION("test empty constructor"){
	    CardStackT newCs = CardStackT(10, std::vector<CardT>());
	   	REQUIRE(newCs.getSize() == 0);
	   	REQUIRE(newCs.getMaxSize() == 10);
	  }


	SECTION("test non-empty constructor"){
		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);
  		

		std::vector<CardT> cards = std::vector<CardT>();
		cards.push_back(c0);cards.push_back(c1);cards.push_back(c2);cards.push_back(c3);

		REQUIRE(cards[0].getSuit() == H);
		REQUIRE(cards[1].getSuit() == D);
		REQUIRE(cards[2].getSuit() == S);
		REQUIRE(cards[3].getSuit() == C);

	    CardStackT cs = CardStackT(10, cards);
	   	REQUIRE(cs.getSize() == 4);
	   	REQUIRE(cs.getMaxSize() == 10);
	   	REQUIRE(cs.top().getRank() == J);
	  }


  	SECTION("test constructor with long sequence throws invalid_argument error"){
  		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);
  		
  		std::vector<CardT> cards = std::vector<CardT>();
  		cards.push_back(c0);cards.push_back(c1);cards.push_back(c2);cards.push_back(c3);

		REQUIRE_NOTHROW(CardStackT(4, cards));
		REQUIRE_THROWS_AS(CardStackT(3, cards), invalid_argument);
  	}

  	SECTION("test constructor with zero maximum size throws invalid_argument"){
  		REQUIRE_THROWS_AS(CardStackT(0, std::vector<CardT>()), invalid_argument);
  	}

	SECTION("test push does not change original stack's size"){
		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);
  		
  		REQUIRE(cs.getSize() == 0);
  		cs.push(c0);
  		REQUIRE(cs.getSize() == 0);
  		cs.push(c1);
  		REQUIRE(cs.getSize() == 0);
  		cs.push(c2);
  		REQUIRE(cs.getSize() == 0);
  		cs.push(c3);
  		REQUIRE(cs.getSize() == 0);
  	}

  	SECTION("test push increases size of new object"){
		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);
  		
  		REQUIRE(cs.getSize() == 0);
  		cs = cs.push(c0);
  		REQUIRE(cs.getSize() == 1);
  		cs = cs.push(c1);
  		REQUIRE(cs.getSize() == 2);
  		cs = cs.push(c2);
  		REQUIRE(cs.getSize() == 3);
  		cs = cs.push(c3);
  		REQUIRE(cs.getSize() == 4);
  	}

  	SECTION("test push changes top of new stack"){
		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);
  		
  		cs = cs.push(c0);
  		REQUIRE(cs.top().getSuit() == H);
		REQUIRE(cs.top().getRank() == C2);
  		cs = cs.push(c1);
  		REQUIRE(cs.top().getSuit() == D);
  		REQUIRE(cs.top().getRank() == C9);
  		cs = cs.push(c2);
  		REQUIRE(cs.top().getSuit() == S);
  		REQUIRE(cs.top().getRank() == K);
  		cs = cs.push(c3);
  		REQUIRE(cs.top().getSuit() == C);
  		REQUIRE(cs.top().getRank() == J);
  	}

  	SECTION("test push does not change top of original stack"){
		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);
  		
  		cs = cs.push(c0);
  		cs = cs.push(c1);
  		cs = cs.push(c3);

  		REQUIRE(cs.top().getSuit() == C);
  		REQUIRE(cs.top().getRank() == J);
  		cs.push(c0);

  		REQUIRE(cs.top().getSuit() == C);
  		REQUIRE(cs.top().getRank() == J);
  		cs.push(c0);  		

  		REQUIRE(cs.top().getSuit() == C);
  		REQUIRE(cs.top().getRank() == J);
  		cs.push(c0);  		

  		REQUIRE(cs.top().getSuit() == C);
  		REQUIRE(cs.top().getRank() == J);
  	}

  	SECTION("test pop does not change original stack's size"){
		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);
  		
  		cs = cs.push(c0);
  		cs = cs.push(c1);
  		cs = cs.push(c2);
  		cs = cs.push(c3);

  		REQUIRE(cs.getSize() == 4);
  		cs.pop();
  		REQUIRE(cs.getSize() == 4);
  		cs.pop();
  		REQUIRE(cs.getSize() == 4);
  		cs.pop();
  		REQUIRE(cs.getSize() == 4);
  		cs.pop();
  		REQUIRE(cs.getSize() == 4);
  	}

  	SECTION("test pop does not change top of original stack"){
		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);
  		
  		cs = cs.push(c0);
  		cs = cs.push(c1);
  		cs = cs.push(c2);
  		cs = cs.push(c3);

  		REQUIRE(cs.top().getSuit() == C);
  		REQUIRE(cs.top().getRank() == J);
  		cs.pop();

  		REQUIRE(cs.top().getSuit() == C);
  		REQUIRE(cs.top().getRank() == J);
  		cs.pop();

  		REQUIRE(cs.top().getSuit() == C);
  		REQUIRE(cs.top().getRank() == J);
  		cs.pop();

  		REQUIRE(cs.top().getSuit() == C);
  		REQUIRE(cs.top().getRank() == J);
  	}

  	SECTION("test pop decreases size of new stack"){
		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);
  		
  		cs = cs.push(c0);
  		cs = cs.push(c1);
  		cs = cs.push(c2);
  		cs = cs.push(c3);

  		REQUIRE(cs.getSize() == 4);
  		cs = cs.pop();
  		REQUIRE(cs.getSize() == 3);
  		cs = cs.pop();
  		REQUIRE(cs.getSize() == 2);
  		cs = cs.pop();
  		REQUIRE(cs.getSize() == 1);
  		cs = cs.pop();
  		REQUIRE(cs.getSize() == 0);
  	}

  	SECTION("test pop changes top of new stack"){
		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);
  		
  		cs = cs.push(c0);
  		cs = cs.push(c1);
  		cs = cs.push(c2);
  		cs = cs.push(c3);

  		REQUIRE(cs.top().getSuit() == C);
  		REQUIRE(cs.top().getRank() == J);
  		cs = cs.pop();

  		REQUIRE(cs.top().getSuit() == S);
  		REQUIRE(cs.top().getRank() == K);
  		cs = cs.pop();

  		REQUIRE(cs.top().getSuit() == D);
  		REQUIRE(cs.top().getRank() == C9);
  		cs = cs.pop();

  		REQUIRE(cs.top().getSuit() == H);
		REQUIRE(cs.top().getRank() == C2);
  	}

  	SECTION("test pop from empty stack throws stack_empty"){
		REQUIRE_THROWS_AS(cs.pop(), stack_empty);
  	}

  	SECTION("test top on empty stack throws stack_empty"){
		REQUIRE_THROWS_AS(cs.top(), stack_empty);
  	}

  	SECTION("test push on full stack throws stack_full"){
  		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);
  		
  		cs = cs.push(c0);
  		cs = cs.push(c1);
  		cs = cs.push(c2);
  		cs = cs.push(c3);

		REQUIRE_THROWS_AS(cs.push(c0), stack_full);
  	}

  	SECTION("test toSeq returns correct sequence"){
  		CardT c0(H, C2);
  		CardT c1(D, C9);
  		CardT c2(S, K);
  		CardT c3(C, J);

  		//proper ordering of cards
		std::vector<CardT> cards = std::vector<CardT>();
		cards.push_back(c0);cards.push_back(c1);cards.push_back(c2);cards.push_back(c3);
  		
  		//test when placed by push()
  		cs = cs.push(c0);
  		cs = cs.push(c1);
  		cs = cs.push(c2);
  		cs = cs.push(c3);

		std::vector<CardT> fromStack = cs.toSeq();
		for (int i = 0; i < fromStack.size(); i++){
			REQUIRE(cards[i].getSuit() == fromStack[i].getSuit());
			REQUIRE(cards[i].getRank() == fromStack[i].getRank());
		}

		//test when placed by constructor
		CardStackT newCs = CardStackT(4, cards);

		std::vector<CardT> fromNewStack = newCs.toSeq();
		for (int i = 0; i < fromNewStack.size(); i++){
			REQUIRE(cards[i].getSuit() == fromNewStack[i].getSuit());
			REQUIRE(cards[i].getRank() == fromNewStack[i].getRank());
		}
  	}

}
